﻿// ===============================================================================
//  解决方案：ConsoleApplication2
//  项目名称：ConsoleApplication2
//  文件名称：Program.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-24 21:37
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;

namespace ConsoleApplication2
{
	internal static class Program
	{
		internal static void Main(string[] args)
		{
			const int times = 20;

			var testData = new List<List<string>> {
				GenerateData(1),
				GenerateData(10),
				GenerateData(100),
				GenerateData(1000),
				GenerateData(10000),
				GenerateData(100000),
				GenerateData(1000000),
				GenerateData(10000000),
			};

			Console.WriteLine("测试数据准备完成，请按任意键继续……");
			Console.ReadKey();

			RunTest(testData, times);

			Console.WriteLine("测试完毕！");
			Console.ReadKey();
		}

		public static void RunTest(List<List<string>> testData, int times)
		{
			const int columnWidth = 14;

			Console.WriteLine("{0} {1} {2}", "itemCount".PadLeft(columnWidth, ' '), "StringBuilder".PadLeft(columnWidth, ' '), "String.Join".PadLeft(columnWidth, ' '));

			foreach (var data in testData)
			{
				var aValue = 0D;
				var bValue = 0D;

				for (var i = 0; i < times; i++)
				{
					aValue += Run(TestA, data);
					bValue += Run(TestB, data);
				}

				Console.WriteLine("{0} {1} {2}", data.Count.ToString("F0").PadLeft(columnWidth, ' '), aValue.ToString("F6").PadLeft(columnWidth, ' '), bValue.ToString("F6").PadLeft(columnWidth, ' '));
			}
		}

		public static double Run(Func<List<string>, string> test, List<string> items)
		{
			var stopwatch = Stopwatch.StartNew();

			test(items);

			stopwatch.Stop();

			return stopwatch.Elapsed.TotalMilliseconds;
		}

		public static List<string> GenerateData(int count)
		{
			var items = new List<string>();
			var r = new Random();

			for (var i = 0; i < count; i++) items.Add(r.Next().ToString(CultureInfo.InvariantCulture));

			return items;
		}

		public static string TestA(List<string> items)
		{
			var length = items.Sum(t => t.Length);

			length += (items.Count - 1);

			if (length == 0) return String.Empty;

			// 先计算出capacity的值
			var builder = new StringBuilder(length);

			builder.Append(items[0]);

			for (var i = 1; i < items.Count; i++)
			{
				builder.Append("|");
				builder.Append(items[i]); // 消灭了之前的一个局部变量，减少内存分配
			}

			return builder.ToString();
		}

		public static string TestB(List<string> items) { return String.Join("|", items); }

		//[SecuritySafeCritical]
		//public static unsafe string Join(string separator, string[] value, int startIndex, int count)
		//{
		//    if (value == null) throw new ArgumentNullException("value");
		//    if (startIndex < 0) throw new ArgumentOutOfRangeException("startIndex", Environment.GetResourceString("ArgumentOutOfRange_StartIndex"));
		//    if (count < 0) throw new ArgumentOutOfRangeException("count", Environment.GetResourceString("ArgumentOutOfRange_NegativeCount"));
		//    if (startIndex > (value.Length - count)) throw new ArgumentOutOfRangeException("startIndex", Environment.GetResourceString("ArgumentOutOfRange_IndexCountBuffer"));
		//    if (separator == null) separator = Empty;
		//    if (count == 0) return Empty;

		//    var length = 0;
		//    var num2 = (startIndex + count) - 1;

		//    for (var i = startIndex; i <= num2; i++) if (value[i] != null) length += value[i].Length;

		//    length += (count - 1)*separator.Length;

		//    if ((length < 0) || ((length + 1) < 0)) throw new OutOfMemoryException();
		//    if (length == 0) return Empty;

		//    string str = FastAllocateString(length);

		//    fixed (char* chRef = &str.m_firstChar)
		//    {
		//        var buffer = new UnSafeCharBuffer(chRef, length);
		//        buffer.AppendString(value[startIndex]);

		//        for (var j = startIndex + 1; j <= num2; j++)
		//        {
		//            buffer.AppendString(separator);
		//            buffer.AppendString(value[j]);
		//        }
		//    }

		//    return str;
		//}
	}
}
